/*
 * Copyright (c) 2018-2028, AoKunSang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.resouces;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.IntStream;

import org.springblade.bi.client.BiTenantClient;
import org.springblade.core.constant.BladeConstant;
import org.springblade.cube.model.CubeDatasouceFunction;
import org.springblade.cube.model.CubeGlobalParam;
import org.springblade.cube.model.CubeGlobalParamType;
import org.springblade.cube.model.CubeWorkspace;
import org.springblade.cube.service.CubeDatasouceFunctionService;
import org.springblade.cube.service.CubeGlobalParamService;
import org.springblade.cube.service.CubeGlobalParamTypeService;
import org.springblade.cube.service.CubeWorkspaceService;
import org.springblade.visual.entity.VisualClassify;
import org.springblade.visual.service.VisualClassifyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.collection.CollectionUtil;
import springfox.documentation.annotations.ApiIgnore;

/**
 * RPC接口实现类
 *
 * @author wookvn
 * @datetime 2025-01-08 15:12:31
 */
@ApiIgnore
@RestController
public class BiTenantResource implements BiTenantClient {

    @Autowired
    private CubeWorkspaceService cubeWorkspaceService;

    @Autowired
    private VisualClassifyService visualClassifyService;

    @Autowired
    private CubeGlobalParamService cubeGlobalParamService;

    @Autowired
    private CubeGlobalParamTypeService cubeGlobalParamTypeService;

    @Autowired
    private CubeDatasouceFunctionService cubeDatasouceFunctionService;

    @Override
    public void tenantInit(String tenantId) {
        init(tenantId);
    }

    @Transactional(rollbackFor = Exception.class)
    private void init(String tenantId) {
        //创建数据集分类
        List<CubeWorkspace> cubeWorkspaceList = new ArrayList<>();
        String[] values = {"默认", "单表表单", "聚合表单"};
        IntStream.range(0, values.length).forEach(index -> {
            CubeWorkspace workspace = new CubeWorkspace();
            workspace.setSortNum(999 - index);
            workspace.setWsName(values[index]);
            workspace.setCreateBy("系统创建");
            workspace.setCreateTime(new Date());
            workspace.setParentId(0);
            workspace.setCreateUserId(0L);
            workspace.setSystemStatus(1);
            workspace.setTenantId(tenantId);
        });
        cubeWorkspaceService.saveBatch(cubeWorkspaceList);
        //创建大屏分类
        VisualClassify classify = new VisualClassify();
        classify.setTenantId(tenantId);
        classify.setCreateTime(new Date());
        classify.setParentId(0);
        classify.setCreateUserId(0L);
        classify.setCategoryValue("默认");
        visualClassifyService.save(classify);
        // 添加全局参数系统变量
        CubeGlobalParamType cubeGlobalParamType = new CubeGlobalParamType();
        cubeGlobalParamType.setCreateTime(LocalDateTime.now());
        cubeGlobalParamType.setCreateUserId(0L);
        cubeGlobalParamType.setParentId(0);
        cubeGlobalParamType.setTypeName("系统变量");
        cubeGlobalParamType.setSortNum(999);
        cubeGlobalParamType.setTenantId(tenantId);
        cubeGlobalParamTypeService.save(cubeGlobalParamType);
        int globalParamTypeId = cubeGlobalParamType.getId();
        List<CubeGlobalParam> globalParamList = cubeGlobalParamService.list(Wrappers.<CubeGlobalParam>lambdaQuery().eq(CubeGlobalParam::getTenantId, BladeConstant.ADMIN_TENANT_ID));
        if(CollectionUtil.isNotEmpty(globalParamList)) {
            globalParamList.stream().forEach(gloalParam -> {
                gloalParam.setId(null);
                gloalParam.setTypeId(globalParamTypeId);
                gloalParam.setTenantId(tenantId);
            });
            cubeGlobalParamService.saveBatch(globalParamList);
        }
        //添加数据源函数
        List<CubeDatasouceFunction> functionList = cubeDatasouceFunctionService.list(Wrappers.<CubeDatasouceFunction>lambdaQuery().eq(CubeDatasouceFunction::getTenantId, BladeConstant.ADMIN_TENANT_ID));
        if(CollectionUtil.isNotEmpty(functionList)) {
            functionList.stream().forEach(item -> {
                item.setFuncId(null);
                item.setTenantId(tenantId);
            });
            cubeDatasouceFunctionService.saveBatch(functionList);
        }
    }
}
